@@ -1,8 +1,9 @@
 /* Generate a file containing some preset patterns.
 
-   Copyright (C) 1995, 1996, 1997, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 2001, 2003 Free Software
+   Foundation, Inc.
 
-   Franois Pinard <pinard@iro.umontreal.ca>, 1995.
+   François Pinard <pinard@iro.umontreal.ca>, 1995.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,7 +22,7 @@
 
 #include "system.h"
 
-#include <copysym.h>
+#include <argmatch.h>
 #include <getopt.h>
 
 #ifndef EXIT_SUCCESS
@@ -33,8 +34,8 @@
 
 enum pattern
 {
-  DEFAULT,
-  ZEROS
+  DEFAULT_PATTERN,
+  ZEROS_PATTERN
 };
 
 /* The name this program was run with. */
@@ -50,7 +51,7 @@ static int show_version = 0;
 static int file_length = 0;
 
 /* Pattern to generate.  */
-static enum pattern pattern = DEFAULT;
+static enum pattern pattern = DEFAULT_PATTERN;
 
 /* Explain how to use the program, then get out.  */
 void
@@ -91,13 +92,8 @@ static const struct option long_options[] =
   {0, 0, 0, 0},
 };
 
-
-const char *pattern_strings[] =
-{
-  "default",			/* 0 */
-  "zeros",			/* 1 */
-  NULL
-};
+static char const * const pattern_args[] = { "default", "zeros", 0 };
+static enum pattern const pattern_types[] = {DEFAULT_PATTERN, ZEROS_PATTERN};
 
 int
 main (int argc, char *const *argv)
@@ -125,25 +121,8 @@ main (int argc, char *const *argv)
 	break;
 
       case 'p':
-	switch (argmatch (optarg, pattern_strings))
-	  {
-
-	  case -2:
-	    error (0, 0, _("Ambiguous pattern `%s'"), optarg);
-	    usage (EXIT_FAILURE);
-
-	  case -1:
-	    error (0, 0, _("Unknown pattern `%s'"), optarg);
-	    usage (EXIT_FAILURE);
-
-	  case 0:
-	    pattern = DEFAULT;
-	    break;
-
-	  case 1:
-	    pattern = ZEROS;
-	    break;
-	  }
+	pattern = XARGMATCH ("--pattern", optarg,
+			     pattern_args, pattern_types);
 	break;
       }
 
@@ -151,17 +130,15 @@ main (int argc, char *const *argv)
 
   if (show_version)
     {
-      char buf[MB_LEN_MAX + 1];
       printf ("genfile (GNU %s) %s\n", PACKAGE, VERSION);
-      printf ("Copyright %s 2001 Free Software Foundation, Inc.\n",
-	      copyright_symbol (buf, sizeof buf));
+      printf (_("Copyright (C) %d Free Software Foundation, Inc.\n"), 2003);
       puts (_("\
 This program comes with NO WARRANTY, to the extent permitted by law.\n\
 You may redistribute it under the terms of the GNU General Public License;\n\
 see the file named COPYING for details."));
 
-      /* Note to translator: Please translate "F. Pinard" to "Franois
-	 Pinard" if "" (c-with-cedilla) is available in the
+      /* Note to translator: Please translate "F. Pinard" to "François
+	 Pinard" if "ç" (c-with-cedilla) is available in the
 	 translation's character set and encoding.  */
       puts (_("Written by F. Pinard."));
 
@@ -178,12 +155,12 @@ see the file named COPYING for details."));
 
   switch (pattern)
     {
-    case DEFAULT:
+    case DEFAULT_PATTERN:
       for (counter = 0; counter < file_length; counter++)
 	putchar (counter & 255);
       break;
 
-    case ZEROS:
+    case ZEROS_PATTERN:
       for (counter = 0; counter < file_length; counter++)
 	putchar (0);
       break;
@@ -191,9 +168,3 @@ see the file named COPYING for details."));
 
   exit (0);
 }
-
-/*
-  Local Variables:
-  coding: iso-latin-1
-  End:
-*/
